#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const prompts_1 = __importDefault(require("prompts"));
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = __importDefault(require("fs"));
const moment_1 = __importDefault(require("moment"));
const yargs_1 = __importDefault(require("yargs"));
const underscore_string_1 = __importDefault(require("underscore.string"));
const logfetcher_1 = __importDefault(require("./lib/logfetcher"));
const logparser_1 = __importDefault(require("./lib/logparser"));
const logemitter_1 = __importDefault(require("./lib/logemitter"));
const { log } = console;
const initialBytesRead = 20000;
const pollingSeconds = 3;
let profiles;
let fileobjs = [];
let profile;
let debug = false;
let run = async function () {
    let packageJson = JSON.parse(fs_1.default.readFileSync(path_1.default.join(__dirname, '../package.json'), 'utf8'));
    log(`cctail - v${packageJson.version} - (c) openmind`);
    readLogConf();
    if (!profiles || Object.keys(profiles).length === 0) {
        readDwJson();
    }
    yargs_1.default.parserConfiguration({
        "parse-numbers": false
    });
    const args = yargs_1.default.argv;
    if (args.d) {
        debug = true;
    }
    if (Object.keys(profiles).length === 1) {
        profile = profiles[Object.keys(profiles)[0]];
    }
    else {
        let profilename = args._[0];
        if (profilename === undefined) {
            const profileselection = await prompts_1.default({
                type: 'select',
                name: 'value',
                message: 'Select a profile:',
                choices: Object.keys(profiles).map(i => ({
                    title: `  [${i}] ${profiles[i].hostname}`,
                    value: `${i}`
                }))
            });
            profilename = profileselection.value;
        }
        if (!profilename) {
            log('No profile selected, exiting\n');
            process.exit(-1);
        }
        if (!profiles[`${profilename}`]) {
            log(chalk_1.default.red(`ERROR: Specified profile ${profilename} not found.\n`));
            process.exit(0);
        }
        profile = profiles[profilename];
    }
    let data = await logfetcher_1.default.fetchLogList(profile);
    let regexp = new RegExp(`<a href="/on/demandware.servlet/webdav/Sites/Logs/(.*?)">[\\s\\S\\&\\?]*?<td align="right">(?:<tt>)?(.*?)(?:<\\/tt>)?</td>[\\s\\S\\&\\?]*?<td align="right"><tt>(.*?)</tt></td>`, 'gim');
    let match = regexp.exec(data);
    while (match != null) {
        // log(`\nparsing "${match[3]} in ${match[0]}"`)
        let filedate = moment_1.default(match[3]);
        if (match[1].substr(-4) === '.log' && filedate.isSame(moment_1.default(), 'day')) {
            fileobjs.push({
                log: match[1],
                sizestring: match[2],
                date: moment_1.default(match[3]),
                debug: debug
            });
        }
        match = regexp.exec(data);
    }
    fileobjs.sort((a, b) => b.date.unix() - a.date.unix());
    let logx = [];
    let logchoiche = [];
    for (let i in fileobjs) {
        let sizeformatted = underscore_string_1.default.lpad(fileobjs[i].sizestring, 12);
        if (sizeformatted.trim() !== '0.0 kb') {
            sizeformatted = chalk_1.default.yellow(sizeformatted);
        }
        let dateformatted = underscore_string_1.default.lpad(fileobjs[i].date.format('YYYY-MM-DD HH:mm:ss'), 20);
        if (fileobjs[i].date.isSame(moment_1.default(), 'hour')) {
            dateformatted = chalk_1.default.yellow(dateformatted);
        }
        let logname = underscore_string_1.default.rpad(fileobjs[i].log, 70);
        logname = colorize(logname, logname);
        logchoiche.push({
            title: `${chalk_1.default.green(underscore_string_1.default.lpad(i, 2))} ${logname} ${sizeformatted}  ${dateformatted}`,
            value: i
        });
    }
    let logselection = await prompts_1.default({
        type: 'autocompleteMultiselect',
        name: 'value',
        message: `Select logs on [${chalk_1.default.green(profile.hostname)}]`,
        choices: logchoiche,
        // eslint-disable-next-line no-return-assign
        onState: ((statedata) => { statedata.value.forEach((i) => i.title = `\n${i.title}`); })
    });
    if (!logselection.value || logselection.value.length === 0) {
        log('No log selected, exiting.\n');
        process.exit(-1);
    }
    logselection.value.forEach((i) => {
        logx.push(fileobjs[i]);
    });
    log('\n');
    // get sizes
    await Promise.all(logx.map(async (logobj) => {
        let size = await logfetcher_1.default.fetchFileSize(profile, logobj);
        logobj.size = Math.max(size - initialBytesRead, 0);
    }));
    setImmediate(showlogs, logx);
};
let showlogs = async function (logx) {
    let parsed = logemitter_1.default.sort(await logparser_1.default.process(logx.map((logobj) => logfetcher_1.default.fetchLogContent(profile, logobj))));
    logemitter_1.default.output(parsed, false, logx[0].debug);
    setTimeout(showlogs, pollingSeconds * 1000, logx);
};
function readDwJson() {
    let dwJsonPath = path_1.default.join(process.cwd(), 'dw.json');
    log(`Loading profile from ${dwJsonPath}\n`);
    try {
        const dwJson = JSON.parse(fs_1.default.readFileSync(dwJsonPath, 'utf8'));
        const name = dwJson.profile || dwJson.hostname.split('-')[0].split('-')[0];
        profiles[name] = dwJson;
    }
    catch (err) {
        log(chalk_1.default.red(`No log.conf.json or dw.json found in path ${process.cwd()}\n`));
        process.exit(-1);
    }
}
function colorize(logname, text) {
    if (underscore_string_1.default.contains(logname, 'error') || underscore_string_1.default.contains(logname, 'fatal')) {
        return chalk_1.default.red(text);
    }
    if (underscore_string_1.default.contains(logname, 'warn')) {
        return chalk_1.default.yellow(text);
    }
    if (underscore_string_1.default.contains(logname, 'info')) {
        return chalk_1.default.green(text);
    }
    if (underscore_string_1.default.contains(logname, 'jobs')) {
        return chalk_1.default.blue(text);
    }
    return text;
}
function readLogConf() {
    try {
        profiles = JSON.parse(fs_1.default.readFileSync(`${process.cwd()}/log.conf.json`, 'utf8'));
    }
    catch (err) {
        // ignore
    }
}
run();
